@@ -1,3 +1,4 @@
+from __future__ import print_function
 from math import *
 from stretch_body.stepper import *
 from stretch_body.device import Device
@@ -9,8 +10,8 @@ class Base(Device):
     """
     API to the Stretch RE1 Mobile Base
     """
-    def __init__(self):
-        Device.__init__(self)
+    def __init__(self,verbose=False):
+        Device.__init__(self,verbose)
         self.name='base'
         self.logger = logging.getLogger('robot.base')
         self.params = self.robot_params[self.name]
@@ -30,8 +31,7 @@ class Base(Device):
     # ###########  Device Methods #############
 
     def startup(self):
-        self.left_wheel.startup()
-        self.right_wheel.startup()
+        return self.left_wheel.startup() and self.right_wheel.startup()
 
 
     def stop(self):
@@ -39,20 +39,20 @@ class Base(Device):
         self.right_wheel.stop()
 
     def pretty_print(self):
-        print '----------Base------'
-        print 'X (m)',self.status['x']
-        print 'Y (m)',self.status['y']
-        print 'Theta (rad)',self.status['theta']
-        print 'X_vel (m/s)', self.status['x_vel']
-        print 'Y_vel (m/s)', self.status['y_vel']
-        print 'Theta_vel (rad/s)', self.status['theta_vel']
-        print 'Pose time (s)', self.status['pose_time_s']
-        print 'Translation Force (N)',self.status['translation_force']
-        print 'Rotation Torque (Nm)', self.status['rotation_torque']
-        print 'Timestamp PC (s):', self.status['timestamp_pc']
-        print '-----Left-Wheel-----'
+        print('----------Base------')
+        print('X (m)',self.status['x'])
+        print('Y (m)',self.status['y'])
+        print('Theta (rad)',self.status['theta'])
+        print('X_vel (m/s)', self.status['x_vel'])
+        print('Y_vel (m/s)', self.status['y_vel'])
+        print('Theta_vel (rad/s)', self.status['theta_vel'])
+        print('Pose time (s)', self.status['pose_time_s'])
+        print('Translation Force (N)',self.status['translation_force'])
+        print('Rotation Torque (Nm)', self.status['rotation_torque'])
+        print('Timestamp PC (s):', self.status['timestamp_pc'])
+        print('-----Left-Wheel-----')
         self.left_wheel.pretty_print()
-        print '-----Right-Wheel-----'
+        print('-----Right-Wheel-----')
         self.right_wheel.pretty_print()
 
     # ###################################################
