@@ -1,13 +1,11 @@
-#! /usr/bin/env python
-from stretch_body.device import Device
+from __future__ import print_function
 import stretch_body.hello_utils as hello_utils
 import time
 
-
 # The code can be found in the following directory:
 # /opt/ros/melodic/lib/python2.7/dist-packages/dynamixel_sdk/
 from dynamixel_sdk.robotis_def import *
-from dynamixel_XL430 import *
+from stretch_body.dynamixel_XL430 import *
 import dynamixel_sdk.port_handler as prh
 import dynamixel_sdk.packet_handler as pch
 import dynamixel_sdk.group_sync_read as gsr
@@ -19,15 +17,24 @@ class DynamixelXChain(Device):
     It allows adding more than one servo at run time
     It allos manage group reading of status data from servos so as to not overload the control bus
     """
-    def __init__(self,usb):
-        Device.__init__(self)
+    def __init__(self,usb,verbose=False):
+        Device.__init__(self,verbose)
         self.usb = usb
         self.timer_stats = hello_utils.TimerStats()
         self.pt_lock = threading.RLock()
-        self.port_handler = prh.PortHandler(usb)
-        self.port_handler.openPort()
-        self.port_handler.setBaudRate(57600)
-        self.packet_handler = pch.PacketHandler(2.0)
+
+        try:
+            self.port_handler = prh.PortHandler(usb)
+            self.port_handler.openPort()
+            self.port_handler.setBaudRate(57600)
+            self.packet_handler = pch.PacketHandler(2.0)
+            self.hw_valid = True
+        except serial.SerialException as e:
+            self.packet_handler = None
+            self.port_handler = None
+            self.hw_valid =False
+            print("SerialException({0}): {1}".format(e.errno, e.strerror))
+
         self.status={}
         self.motors = {}
         self.readers={}
@@ -38,6 +45,8 @@ class DynamixelXChain(Device):
         self.motors[m.name]=m
 
     def startup(self):
+        if not self.hw_valid:
+            return False
         if len(self.motors.keys()):
             self.readers['pos']=gsr.GroupSyncRead(self.port_handler, self.packet_handler, XL430_ADDR_PRESENT_POSITION,4)
             self.readers['effort']=gsr.GroupSyncRead(self.port_handler, self.packet_handler, XL430_ADDR_PRESENT_LOAD,2)
@@ -51,13 +60,18 @@ class DynamixelXChain(Device):
                 self.motors[mk].startup()
                 self.status[mk]=self.motors[mk].status
             self.pull_status()
+        return True
 
     def stop(self):
+        if not self.hw_valid:
+            return
         for mk in self.motors.keys():
             self.motors[mk].stop()
 
 
     def pull_status(self):
+        if not self.hw_valid:
+            return
         try:
             ts = time.time()
             pos = self.sync_read(self.readers['pos'])
@@ -72,16 +86,18 @@ class DynamixelXChain(Device):
                 idx=idx+1
             self.timer_stats.update(time.time()-ts)
         except IOError:
-            print 'IOError on:',self.usb
+            print('IOError on:',self.usb)
 
     def pretty_print(self):
-        print '--- Dynamixel X Chain ---'
-        print 'USB', self.usb
+        print('--- Dynamixel X Chain ---')
+        print('USB', self.usb)
         self.timer_stats.pretty_print()
         for mk in self.motors.keys():
             self.motors[mk].pretty_print()
 
     def sync_read(self, reader):
+        if not self.hw_valid:
+            return
         with self.pt_lock:
             result = reader.txRxPacket()
         if result != COMM_SUCCESS:
@@ -114,17 +130,19 @@ class DynamixelXChain(Device):
 
 
     def step_sentry(self,runstop):
+        if not self.hw_valid:
+            return
         """
         This sentry places the Dynamixel servos in torque_disabled
         mode when the runstop is enabled
         """
         if runstop is not self.runstop_last:
             if runstop:
-                #print 'Disabling torque to ',self.name
+                #print('Disabling torque to ',self.name)
                 for mk in self.motors.keys():
                     self.motors[mk].disable_torque()
             else:
-                #print 'Enabling torque to ', self.name
+                #print('Enabling torque to ', self.name)
                 for mk in self.motors.keys():
                     self.motors[mk].enable_torque()
 
