@@ -1,5 +1,5 @@
-
-from dynamixel_XL430 import *
+from __future__ import print_function
+from stretch_body.dynamixel_XL430 import *
 from stretch_body.device import Device
 import time
 from stretch_body.hello_utils import *
@@ -9,10 +9,9 @@ class DynamixelHelloXL430(Device):
     """
     Abstract the Dynamixel X-Series to handle calibration, radians, etc
     """
-    def __init__(self,name, chain=None):
-        Device.__init__(self)
+    def __init__(self,name, chain=None,verbose=False):
+        Device.__init__(self,verbose)
         self.name=name
-        self.chain=None
         self.params=self.robot_params[self.name]
         self.status={'timestamp_pc':0,'comm_errors':0,'pos':0,'vel':0,'effort':0,'temp':0,'shutdown':0, 'hardware_error':0,
                      'input_voltage_error':0,'overheating_error':0,'motor_encoder_error':0,'electrical_shock_error':0,'overload_error':0,
@@ -27,17 +26,17 @@ class DynamixelHelloXL430(Device):
         else:
             self.polarity=1.0
         self.ts_over_eff_start=None
-        self.servo_valid=False
+        self.hw_valid=False
         self.is_calibrated=False
 
     # ###########  Device Methods #############
 
-    def do_ping(self, verbose):
+    def do_ping(self, verbose=False):
         return self.motor.do_ping(verbose)
 
     def startup(self):
         if self.motor.do_ping(verbose=False):
-            self.servo_valid = True
+            self.hw_valid = True
             self.motor.disable_torque()
             if self.params['use_multiturn']:
                 self.motor.enable_multiturn()
@@ -55,16 +54,18 @@ class DynamixelHelloXL430(Device):
             self.a_des=self.params['motion']['default']['accel']
             self.is_calibrated=self.motor.is_calibrated()
             self.enable_torque()
+            return True
         else:
-            print 'DynamixelHelloXL430 Ping failed...', self.name
+            print('DynamixelHelloXL430 Ping failed...', self.name)
+            return False
 
 
     def stop(self):
-        if self.servo_valid:
+        if self.hw_valid:
             self.disable_torque()
 
     def pull_status(self,data=None):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         #First pull new data from servo
         #Or bring in data from a synchronized read
@@ -77,7 +78,7 @@ class DynamixelHelloXL430(Device):
                 ts = time.time()
                 err=self.motor.get_hardware_error()
             except termios.error:
-                print 'Dynamixel communiation error at time: ',time.time()
+                print('Dynamixel communiation error at time: ',time.time())
         else:
             x = data['x']
             v = data['v']
@@ -120,67 +121,67 @@ class DynamixelHelloXL430(Device):
             self.status['stall_overload'] = False
 
     def mark_zero(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         x=self.motor.get_pos()
-        print 'Marking current position of (ticks)',x,' as 0 (rad)'
+        print('Marking current position of (ticks)',x,' as 0 (rad)')
         self.params['zero_t']=x
         self.write_device_params(self.name, self.params)
 
 
 
     def pretty_print(self):
-        if not self.servo_valid:
-            print '----- HelloXL430 ------ '
-            print 'Servo not on bus'
+        if not self.hw_valid:
+            print('----- HelloXL430 ------ ')
+            print('Servo not on bus')
             return
-        print '----- HelloXL430 ------ '
-        print 'Name',self.name
-        print 'Position (rad)', self.status['pos']
-        print 'Position (deg)', rad_to_deg(self.status['pos'])
-        print 'Position (ticks)', self.status['pos_ticks']
-        print 'Velocity (rad/s)', self.status['vel']
-        print 'Velocity (ticks/s)', self.status['vel_ticks']
-        print 'Effort (%)', self.status['effort']
-        print 'Effort (ticks)', self.status['effort_ticks']
-        print 'Temp', self.status['temp']
-        print 'Comm Errors', self.motor.comm_errors
-        print 'Hardware Error', self.status['hardware_error']
-        print 'Hardware Error: Input Voltage Error: ',self.status['input_voltage_error']
-        print 'Hardware Error: Overheating Error: ', self.status['overheating_error']
-        print 'Hardware Error: Motor Encoder Error: ',self.status['motor_encoder_error']
-        print 'Hardware Error: Electrical Shock Error: ', self.status['electrical_shock_error']
-        print 'Hardware Error: Overload Error: ', self.status['overload_error']
-        print 'Timestamp PC', self.status['timestamp_pc']
-        print 'Range (ticks)',self.params['range_t']
-        print 'Range (rad) [',self.ticks_to_world_rad(self.params['range_t'][0]), ' , ',self.ticks_to_world_rad(self.params['range_t'][1]),']'
-        print 'Stalled',self.status['stalled']
-        print 'Stall Overload',self.status['stall_overload']
-        print 'Is Calibrated',self.is_calibrated
+        print('----- HelloXL430 ------ ')
+        print('Name',self.name)
+        print('Position (rad)', self.status['pos'])
+        print('Position (deg)', rad_to_deg(self.status['pos']))
+        print('Position (ticks)', self.status['pos_ticks'])
+        print('Velocity (rad/s)', self.status['vel'])
+        print('Velocity (ticks/s)', self.status['vel_ticks'])
+        print('Effort (%)', self.status['effort'])
+        print('Effort (ticks)', self.status['effort_ticks'])
+        print('Temp', self.status['temp'])
+        print('Comm Errors', self.motor.comm_errors)
+        print('Hardware Error', self.status['hardware_error'])
+        print('Hardware Error: Input Voltage Error: ',self.status['input_voltage_error'])
+        print('Hardware Error: Overheating Error: ', self.status['overheating_error'])
+        print('Hardware Error: Motor Encoder Error: ',self.status['motor_encoder_error'])
+        print('Hardware Error: Electrical Shock Error: ', self.status['electrical_shock_error'])
+        print('Hardware Error: Overload Error: ', self.status['overload_error'])
+        print('Timestamp PC', self.status['timestamp_pc'])
+        print('Range (ticks)',self.params['range_t'])
+        print('Range (rad) [',self.ticks_to_world_rad(self.params['range_t'][0]), ' , ',self.ticks_to_world_rad(self.params['range_t'][1]),']')
+        print('Stalled',self.status['stalled'])
+        print('Stall Overload',self.status['stall_overload'])
+        print('Is Calibrated',self.is_calibrated)
         #self.motor.pretty_print()
 
     # #####################################
 
     def reboot(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.do_reboot()
 
     def enable_torque(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.enable_torque()
 
     def disable_torque(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.disable_torque()
 
     def move_to(self,x_des, v_des=None, a_des=None):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         if self.params['req_calibration'] and not self.is_calibrated:
-            print 'Dynamixel not calibrated:', self.name
+            print('Dynamixel not calibrated:', self.name)
             return
         try:
             self.set_motion_params(v_des,a_des)
@@ -188,11 +189,11 @@ class DynamixelHelloXL430(Device):
             t_des = max(self.params['range_t'][0], min(self.params['range_t'][1], t_des))
             self.motor.go_to_pos(t_des)
         except termios.error:
-            print 'Dynamixel communication error at time: ',time.time()
+            print('Dynamixel communication error at time: ',time.time())
 
 
     def set_motion_params(self,v_des=None,a_des=None):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         if v_des is not None:
             v_des = min(self.params['motion']['max']['vel'], v_des)
@@ -207,20 +208,20 @@ class DynamixelHelloXL430(Device):
                 self.a_des = a_des
 
     def move_by(self,x_des, v_des=None, a_des=None):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         if abs(x_des) > 0.00002: #Avoid drift
             cx=self.ticks_to_world_rad(self.motor.get_pos())
             self.move_to(cx + x_des, v_des, a_des)
 
     def quick_stop(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
-            self.motor.disable_torque()
-            self.motor.enable_torque()
+        self.motor.disable_torque()
+        self.motor.enable_torque()
 
     def enable_pos(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.disable_torque()
         if self.params['use_multiturn']:
@@ -230,14 +231,14 @@ class DynamixelHelloXL430(Device):
         self.motor.enable_torque()
 
     def enable_pwm(self):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.disable_torque()
         self.motor.enable_pwm()
         self.motor.enable_torque()
 
     def set_pwm(self,x):
-        if not self.servo_valid:
+        if not self.hw_valid:
             return
         self.motor.set_pwm(x)
 
@@ -248,7 +249,8 @@ class DynamixelHelloXL430(Device):
         # Can be in multiturn or single turn mode
         # Mark the first hardstop as zero ticks on the Dynammixel
         # Second hardstop is optional
-        if not self.servo_valid:
+        if not self.hw_valid:
+            print('Not able to home %s. Hardware not present'%self.name)
             return
         if not self.params['req_calibration']:
             print('Homing not required for: '+self.name)
@@ -256,12 +258,12 @@ class DynamixelHelloXL430(Device):
 
         self.pull_status()
         if self.status['overload_error'] or self.status['overheating_error']:
-            print 'Hardware error, unable to home. Exiting'
+            print('Hardware error, unable to home. Exiting')
             return
 
         self.enable_pwm()
 
-        print 'Moving to first hardstop...'
+        print('Moving to first hardstop...')
         self.set_pwm(self.params['pwm_homing'][0])
         ts=time.time()
         time.sleep(1.0)
@@ -269,20 +271,20 @@ class DynamixelHelloXL430(Device):
         while self.motor.is_moving() and not timeout:
             timeout=time.time()-ts>15.0
             time.sleep(0.5)
-            #print 'Pos (ticks)',self.motor.get_pos()
+            #print('Pos (ticks)',self.motor.get_pos())
         time.sleep(delay_at_stop)
         xs=self.motor.get_pos()
         self.set_pwm(0)
 
         if timeout:
-            print 'Timed out moving to first hardstop. Exiting.'
+            print('Timed out moving to first hardstop. Exiting.')
             return
         if self.status['overload_error'] or self.status['overheating_error']:
-            print 'Hardware error, unable to home. Exiting'
+            print('Hardware error, unable to home. Exiting')
             return
 
-        print 'Contact at position:', xs
-        print 'Hit first hardstop, marking to zero ticks'
+        print('Contact at position:', xs)
+        print('Hit first hardstop, marking to zero ticks')
         self.motor.disable_torque()
         self.motor.zero_position(verbose=False)
         self.motor.set_calibrated(1)
@@ -291,11 +293,11 @@ class DynamixelHelloXL430(Device):
 
         self.enable_pwm()
 
-        print 'Raw position:',self.motor.get_pos()
+        print('Raw position:',self.motor.get_pos())
 
         if not single_stop:
             #Measure the range and write to YAML
-            print 'Moving to second hardstop...'
+            print('Moving to second hardstop...')
             self.set_pwm(self.params['pwm_homing'][1])
             ts = time.time()
             time.sleep(1.0)
@@ -303,31 +305,31 @@ class DynamixelHelloXL430(Device):
             while self.motor.is_moving() and not timeout:
                 timeout = time.time() - ts > 15.0
                 time.sleep(0.5)
-                #print 'Pos (ticks)', self.motor.get_pos()
+                #print('Pos (ticks)', self.motor.get_pos())
             time.sleep(delay_at_stop)
             x_dir_1 = self.motor.get_pos()
             self.set_pwm(0)
 
             if timeout:
-                print 'Timed out moving to second hardstop. Exiting.'
+                print('Timed out moving to second hardstop. Exiting.')
                 return
-            print 'Hit second hardstop at: ', x_dir_1
+            print('Hit second hardstop at: ', x_dir_1)
             if self.status['overload_error'] or self.status['overheating_error']:
-                print 'Hardware error, unable to home. Exiting'
+                print('Hardware error, unable to home. Exiting')
                 return
 
             self.params['range_t']=[0,x_dir_1]
-            print 'Homed to range of:'
-            print '   Ticks:',self.params['range_t']
+            print('Homed to range of:')
+            print('   Ticks:',self.params['range_t'])
             r1=self.ticks_to_world_rad(0)
             r2=self.ticks_to_world_rad(x_dir_1)
-            print '   Radians:',[r1,r2]
-            print '   Degrees:',[rad_to_deg(r1),rad_to_deg(r2)]
+            print('   Radians:',[r1,r2])
+            print('   Degrees:',[rad_to_deg(r1),rad_to_deg(r2)])
             self.write_device_params(self.name, self.params)
 
         self.enable_pos()
         if move_to_zero:
-            print 'Moving to calibrated zero: (rad)'
+            print('Moving to calibrated zero: (rad)')
             self.move_to(0)
             time.sleep(3.0)
 # ##########################################
