@@ -1,6 +1,5 @@
+from __future__ import print_function
 from stretch_body.dynamixel_hello_XL430 import DynamixelHelloXL430
-import time
-from stretch_body.hello_utils import *
 import logging
 
 class StretchGripper(DynamixelHelloXL430):
@@ -11,8 +10,8 @@ class StretchGripper(DynamixelHelloXL430):
     The Pct ranges from approximately -100 (fully closed) to approximately +50 (fully open)
     A Pct of zero is the fingertips just touching
     """
-    def __init__(self, chain=None):
-        DynamixelHelloXL430.__init__(self,'stretch_gripper',chain)
+    def __init__(self, chain=None,verbose=False):
+        DynamixelHelloXL430.__init__(self,'stretch_gripper',chain,verbose=verbose)
         self.logger = logging.getLogger('robot.stretch_gripper')
         self.status['pos_pct']= 0.0
         self.poses = {'zero':0,'open': 50, 'close': -100}
@@ -21,8 +20,8 @@ class StretchGripper(DynamixelHelloXL430):
         DynamixelHelloXL430.home(self,single_stop=True,move_to_zero=move_to_zero,delay_at_stop=3.0)
 
     def pretty_print(self):
-        print '--- StretchGripper ----'
-        print "Position (%)",self.status['pos_pct']
+        print('--- StretchGripper ----')
+        print("Position (%)",self.status['pos_pct'])
         DynamixelHelloXL430.pretty_print(self)
 
     def pose(self,p,v_r=None, a_r=None):
